#ifdef _NES_MAPPER_CPP_

/////////////////////////////////////////////////////////////////////
// Mapper 13
void NES_mapper13::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,2,3);

  // set PPU bank pointers
  parent_NES->ppu->PPU_VRAM_banks[0] = chr_ram + 0x400 * 0;
  parent_NES->ppu->PPU_VRAM_banks[1] = chr_ram + 0x400 * 1;
  parent_NES->ppu->PPU_VRAM_banks[2] = chr_ram + 0x400 * 2;
  parent_NES->ppu->PPU_VRAM_banks[3] = chr_ram + 0x400 * 3;
  parent_NES->ppu->PPU_VRAM_banks[4] = chr_ram + 0x400 * 0;
  parent_NES->ppu->PPU_VRAM_banks[5] = chr_ram + 0x400 * 1;
  parent_NES->ppu->PPU_VRAM_banks[6] = chr_ram + 0x400 * 2;
  parent_NES->ppu->PPU_VRAM_banks[7] = chr_ram + 0x400 * 3;
}

void NES_mapper13::MemoryWrite(uint32 addr, uint8 data)
{
  uint8 prg_bank = (data & 0x30) >> 4;
  uint8 chr_bank = data & 0x03;

  set_CPU_bank4(prg_bank*4);
  set_CPU_bank5(prg_bank*4+1);
  set_CPU_bank6(prg_bank*4+2);
  set_CPU_bank7(prg_bank*4+3);

  parent_NES->ppu->PPU_VRAM_banks[4] = chr_ram + 0x400 * (chr_bank*4);
  parent_NES->ppu->PPU_VRAM_banks[5] = chr_ram + 0x400 * (chr_bank*4+1);
  parent_NES->ppu->PPU_VRAM_banks[6] = chr_ram + 0x400 * (chr_bank*4+2);
  parent_NES->ppu->PPU_VRAM_banks[7] = chr_ram + 0x400 * (chr_bank*4+3);
}

/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Mapper 100
void NES_mapper100::Reset()
{
  // clear registers FIRST!!!
  for(int i = 0; i < 8; i++) regs[i] = 0x00;

  // set CPU bank pointers
  prg0 = 0;
  prg1 = 1;
  prg2 = num_8k_ROM_banks - 2;
  prg3 = num_8k_ROM_banks - 1;
  MMC3_set_CPU_banks();

  // set VROM banks
  if(num_1k_VROM_banks)
  {
    chr0 = 0;
    chr1 = 1;
    chr2 = 2;
    chr3 = 3;
    chr4 = 4;
    chr5 = 5;
    chr6 = 6;
    chr7 = 7;
    MMC3_set_PPU_banks();
  }
  else
  {
    chr0 = chr2 = chr4 = chr5 = chr6 = chr7 = 0;
    chr1 = chr3 = 1;
  }

  irq_enabled = 0;
  irq_counter = 0;
  irq_latch = 0;
}

void NES_mapper100::MemoryWrite(uint32 addr, uint8 data)
{
  switch(addr & 0xE001)
  {
    case 0x8000:
      {
        regs[0] = data;
      }
      break;

    case 0x8001:
      {
        uint32 bank_num;
        regs[1] = data;
        bank_num = regs[1];

        switch(regs[0] & 0xC7)
        {
          case 0x00:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr0 = bank_num;
                chr1 = bank_num + 1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x01:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr2 = bank_num;
                chr3 = bank_num + 1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x02:
            {
              if(num_1k_VROM_banks)
              {
                chr4 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x03:
            {
              if(num_1k_VROM_banks)
              {
                chr5 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x04:
            {
              if(num_1k_VROM_banks)
              {
                chr6 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x05:
            {
              if(num_1k_VROM_banks)
              {
                chr7 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x06:
            {
              prg0 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;

          case 0x07:
            {
              prg1 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;

          case 0x46:
            {
              prg2 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;

          case 0x47:
            {
              prg3 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;

          case 0x80:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr4 = bank_num;
                chr5 = bank_num + 1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x81:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr6 = bank_num;
                chr7 = bank_num + 1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x82:
            {
              if(num_1k_VROM_banks)
              {
                chr0 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x83:
            {
              if(num_1k_VROM_banks)
              {
                chr1 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x84:
            {
              if(num_1k_VROM_banks)
              {
                chr2 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x85:
            {
              if(num_1k_VROM_banks)
              {
                chr3 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;
        }
      }
      break;

    case 0xA000:
      {
        regs[2] = data;

        if(parent_NES->ROM->get_mirroring() != NES_PPU::MIRROR_FOUR_SCREEN)
        {
          if(data & 0x01)
          {
            set_mirroring(NES_PPU::MIRROR_HORIZ);
          }
          else
          {
            set_mirroring(NES_PPU::MIRROR_VERT);
          }
        }
      }
      break;

    case 0xA001:
      {
        regs[3] = data;
      }
      break;

    case 0xC000:
      regs[4] = data;
      irq_counter = regs[4];
      break;

    case 0xC001:
      regs[5] = data;
      irq_latch = regs[5];
      break;

    case 0xE000:
      regs[6] = data;
      irq_enabled = 0;
      break;

    case 0xE001:
      regs[7] = data;
      irq_enabled = 1;
      break;
  }
}

void NES_mapper100::HSync(uint32 scanline)
{
  if(irq_enabled)
  {
    if((scanline >= 0) && (scanline <= 239))
    {
      if(parent_NES->ppu->spr_enabled() || parent_NES->ppu->bg_enabled())
      {
        if(!(irq_counter--))
        {
          irq_counter = irq_latch;
          parent_NES->cpu->DoIRQ();
        }
      }
    }
  }
}
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Mapper 118
void NES_mapper118::Reset()
{
  // clear registers FIRST!!!
  for(int i = 0; i < 8; i++) regs[i] = 0x00;

  // set CPU bank pointers
  prg0 = 0;
  prg1 = 1;
  MMC3_set_CPU_banks();

  // set VROM banks
  if(num_1k_VROM_banks)
  {
    chr0 = 0;
    chr1 = 1;
    chr2 = 2;
    chr3 = 3;
    chr4 = 4;
    chr5 = 5;
    chr6 = 6;
    chr7 = 7;
    MMC3_set_PPU_banks();
  }
  else
  {
    chr0 = chr1 = chr2 = chr3 = chr4 = chr5 = chr6 = chr7 = 0;
  }

  irq_enabled = 0;
  irq_counter = 0;
  irq_latch = 0;
}

void NES_mapper118::MemoryWrite(uint32 addr, uint8 data)
{
  switch(addr & 0xE001)
  {
    case 0x8000:
      {
        regs[0] = data;
        MMC3_set_PPU_banks();
        MMC3_set_CPU_banks();
      }
      break;

    case 0x8001:
      {
        uint32 bank_num;
        regs[1] = data;
        bank_num = regs[1];

        if((regs[0] & 0x07) < 6)
        {
          if(data & 0x80)
          {
            set_mirroring(0,0,0,0);
          }
          else
          {
            set_mirroring(1,1,1,1);
          }
        }

        switch(regs[0] & 0x07)
        {
          case 0x00:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr0 = bank_num;
                chr1 = bank_num+1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x01:
            {
              if(num_1k_VROM_banks)
              {
                bank_num &= 0xfe;
                chr2 = bank_num;
                chr3 = bank_num+1;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x02:
            {
              if(num_1k_VROM_banks)
              {
                chr4 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x03:
            {
              if(num_1k_VROM_banks)
              {
                chr5 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x04:
            {
              if(num_1k_VROM_banks)
              {
                chr6 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x05:
            {
              if(num_1k_VROM_banks)
              {
                chr7 = bank_num;
                MMC3_set_PPU_banks();
              }
            }
            break;

          case 0x06:
            {
              prg0 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;

          case 0x07:
            {
              prg1 = bank_num;
              MMC3_set_CPU_banks();
            }
            break;
        }
      }
      break;

    case 0xC000:
      regs[4] = data;
      irq_counter = regs[4];
      break;

    case 0xC001:
      regs[5] = data;
      irq_latch = regs[5];
      break;

    case 0xE000:
      regs[6] = data;
      irq_enabled = 0;
      break;

    case 0xE001:
      regs[7] = data;
      irq_enabled = 1;
      break;
  }
}

void NES_mapper118::HSync(uint32 scanline)
{
  if(irq_enabled)
  {
    if((scanline >= 0) && (scanline <= 239))
    {
      if(parent_NES->ppu->spr_enabled() || parent_NES->ppu->bg_enabled())
      {
        if(!(irq_counter--))
        {
          irq_counter = irq_latch;
          parent_NES->cpu->DoIRQ();
        }
      }
    }
  }
}

#define MAP118_ROM(ptr)  (((ptr)-parent_NES->ROM->get_ROM_banks())  >> 13)
#define MAP118_VROM(ptr) (((ptr)-parent_NES->ROM->get_VROM_banks()) >> 10)

void NES_mapper118::SNSS_fixup()
{
  NES_6502::Context context;
  parent_NES->cpu->GetContext(&context);

  prg0 = MAP118_ROM(context.mem_page[prg_swap() ? 6 : 4]);
  prg1 = MAP118_ROM(context.mem_page[5]);
  if(num_1k_VROM_banks)
  {
    if(chr_swap())
    {
      chr0 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[4]);
      chr1 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[5]);
      chr2 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[6]);
      chr3 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[7]);
      chr4 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[0]);
      chr5 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[1]);
      chr6 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[2]);
      chr7 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[3]);
    }
    else
    {
      chr0 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[0]);
      chr1 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[1]);
      chr2 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[2]);
      chr3 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[3]);
      chr4 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[4]);
      chr5 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[5]);
      chr6 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[6]);
      chr7 = MAP118_VROM(parent_NES->ppu->PPU_VRAM_banks[7]);
    }
  }
}
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Mapper 240
void NES_mapper240::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,num_8k_ROM_banks-2,num_8k_ROM_banks-1);

  // set PPU bank pointers
  if(num_1k_VROM_banks)
  {
    set_PPU_banks(0,1,2,3,4,5,6,7);
  }
}

void NES_mapper240::MemoryWriteLow(uint32 addr, uint8 data)
{
  if(addr >= 0x4100 && addr <= 0x4FFF)
  {
    uint8 prg_bank = data >> 4;
    uint8 chr_bank = data & 0x0F;

    set_CPU_bank4(prg_bank*4+0);
    set_CPU_bank5(prg_bank*4+1);
    set_CPU_bank6(prg_bank*4+2);
    set_CPU_bank7(prg_bank*4+3);

    set_PPU_bank0(chr_bank*8+0);
    set_PPU_bank1(chr_bank*8+1);
    set_PPU_bank2(chr_bank*8+2);
    set_PPU_bank3(chr_bank*8+3);
    set_PPU_bank4(chr_bank*8+4);
    set_PPU_bank5(chr_bank*8+5);
    set_PPU_bank6(chr_bank*8+6);
    set_PPU_bank7(chr_bank*8+7);
  }
}
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Mapper 242
void NES_mapper242::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,2,3);

  // set PPU bank pointers
  if(num_1k_VROM_banks)
  {
    set_PPU_banks(0,1,2,3,4,5,6,7);
  }
}

void NES_mapper242::MemoryWrite(uint32 addr, uint8 data)
{
  if(addr & 0x0001)
  {
    set_CPU_bank4(((addr & 0x78) >> 1) + 0);
    set_CPU_bank5(((addr & 0x78) >> 1) + 1);
    set_CPU_bank6(((addr & 0x78) >> 1) + 2);
    set_CPU_bank7(((addr & 0x78) >> 1) + 3);
  }
}
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Mapper 246
void NES_mapper246::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,num_8k_ROM_banks-2,num_8k_ROM_banks-1);
}

void NES_mapper246::MemoryWriteSaveRAM(uint32 addr, uint8 data)
{
  switch(addr)
  {
    case 0x6000:
      {
        set_CPU_bank4(data);
      }
      break;

    case 0x6001:
      {
        set_CPU_bank5(data);
      }
      break;

    case 0x6002:
      {
        set_CPU_bank6(data);
      }
      break;

    case 0x6003:
      {
        set_CPU_bank7(data);
      }
      break;

    case 0x6004:
      {
        set_PPU_bank0(data*2+0);
        set_PPU_bank1(data*2+1);
      }
      break;

    case 0x6005:
      {
        set_PPU_bank2(data*2+0);
        set_PPU_bank3(data*2+1);
      }
      break;

    case 0x6006:
      {
        set_PPU_bank4(data*2+0);
        set_PPU_bank5(data*2+1);
      }
      break;

    case 0x6007:
      {
        set_PPU_bank6(data*2+0);
        set_PPU_bank7(data*2+1);
      }
      break;
  }
}
/////////////////////////////////////////////////////////////////////

#endif
